package cn.sh.stc.sict.theme.hpgp.controller.mobile;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.core.util.XmlUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.sh.stc.sict.cloud.common.core.util.R;
import cn.sh.stc.sict.cloud.common.log.annotation.SysLog;
import cn.sh.stc.sict.cloud.common.security.util.SecurityUtils;
import cn.sh.stc.sict.cloud.upms.dto.CurrentUser;
import cn.sh.stc.sict.theme.hpgp.vo.IntelligentAnswerVO;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.google.api.client.util.Lists;
import com.google.common.collect.Sets;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.reactive.function.client.WebClient;

import java.util.*;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;

/**
 * @author F_xh
 */
@Slf4j
@RestController
@Api(tags = "[C]黄浦精准预约——智能问答")
@RequestMapping("/hpgp/c/intelligent/qa")
@RequiredArgsConstructor
public class HpGpIntelligentQAController {
    private final static String CONTENT = "Content";

    @Value("${hpgp-guide.qa-url:http://180.169.131.147:8000/}")
    private String qaUrl;

    @Value("${hpgp-guide.qa-history-limit:3}")
    private Integer qaHistoryLimit;

    private final static String QA_RESULT_FLAG = "Helper: ";
    // 报告查询、医疗知识问答、医疗分诊、预约挂号
    private static final Set<String> QA_TYPE= Sets.newHashSet("报告查询","预约挂号");

    /**
     * 打浦桥科室
     */
    private final static Set<String> DPQ_DEPARTMENT = Sets.newHashSet("皮肤性病科", "中医科", "儿保科", "全科医疗科");
    private final static Set<String> XDM_DEPARTMENT = Sets.newHashSet("全科", "全科名医", "中医", "针伤科", "齿科");

    @SysLog
    @ApiOperation("智能问答")
    @PostMapping
    public R<?> qa(String question) {
        CurrentUser current = SecurityUtils.getCurrentUser();
        String from = StrUtil.isNotBlank(current.getOpenId()) ? current.getOpenId() : current.getId().toString();

        String xml = "<xml>\n" +
                "  <ToUserName><![CDATA[huangpurobot]]></ToUserName>\n" +
                "  <FromUserName><![CDATA[" + from + "]]></FromUserName>\n" +
                "  <CreateTime>" + DateUtil.now() + "</CreateTime>\n" +
                "  <MsgType><![CDATA[text]]></MsgType>\n" +
                "  <Content><![CDATA[" + question + "]]></Content>\n" +
                "  <MsgId>" + IdWorker.getId() + "</MsgId>\n" +
                "</xml>";
        // 162 为黄浦高血压专病导医机器人
        // 51 黄浦高血压全科导诊机器人
        String body = HttpRequest.post("http://30.30.5.74:9988/qa/jqrsvr.ashx?Command=talk_51")
                .body(xml)
                .execute()
                .body();
        log.error("孙总接口请求 xml = {}", xml);
        log.error("孙总接口返回 body = {}", body);
        IntelligentAnswerVO answerVO = new IntelligentAnswerVO();
        Map<String, Object> result = XmlUtil.xmlToMap(body);
        if (result.containsKey(CONTENT)) {
            answerVO.setAnswer(MapUtil.getStr(result, CONTENT));

            // 打浦桥单独处理
            if (StrUtil.isNotBlank(current.getHospitalCode())
                    && "42502942300".equals(current.getHospitalCode())) {
                if (StrUtil.isNotBlank(answerVO.getDeptName())
                        && !DPQ_DEPARTMENT.contains(answerVO.getDeptName())) {
                    String oldDeptName = answerVO.getDeptName();
                    answerVO.setAnswer(answerVO.getAnswer().replace(oldDeptName, "全科医疗科"));
                    answerVO.setDeptName("全科医疗科");
                }
            }

            // 小东门单独处理
            if (StrUtil.isNotBlank(current.getHospitalCode())
                    && "42507025800".equals(current.getHospitalCode())) {
                if (StrUtil.isNotBlank(answerVO.getDeptName())
                        && !XDM_DEPARTMENT.contains(answerVO.getDeptName())) {
                    String oldDeptName = answerVO.getDeptName();
                    answerVO.setAnswer(answerVO.getAnswer().replace(oldDeptName, "全科"));
                    answerVO.setDeptName("全科");
                }
            }

        }
        return new R<>(answerVO);
    }

//    @SysLog
//    @ApiOperation("智能问答")
//    @PostMapping
//    public R qaV2(String question) {
//        if (StrUtil.isNotBlank(question) && question.contains("上周的血检报告出了")) {
//            IntelligentAnswerVO answerVO = new IntelligentAnswerVO();
//            answerVO.setAnswer("<a href=\"https://www.sh-sict.com/healthExam\">https://www.sh-sict.com/healthExam</a>");
//            return new R(answerVO);
//        }
//        CurrentUser current = SecurityUtils.getCurrentUser();
//        List<String> qaHistory = JSON.parseArray(current.getQaHistory(), String.class);
//        String from = current.getId().toString();
//
//        JSONObject jsonObject = new JSONObject();
//        jsonObject.put("username", from);
//        if (CollUtil.isEmpty(qaHistory)) {
//            qaHistory = Lists.newArrayListWithCapacity(1);
//        }
//
//        if (qaHistory.size() > qaHistoryLimit) {
//            qaHistory = qaHistory.subList(0, qaHistoryLimit - 1);
//        }
//
//        List<String> currentQa = Lists.newArrayList();
//        currentQa.add(question);
//        currentQa.add(null);
//        qaHistory.add(JSON.toJSONString(currentQa));
//
//        List<List<String>> reqList = new ArrayList<>();
//        for (Object item : qaHistory) {
//            if (item instanceof String) {
//                List<String> itemList = JSONArray.parseArray((String) item, String.class);
//                reqList.add(itemList);
//            }
//        }
//        jsonObject.put("history", reqList);
//        String req = JSON.toJSONString(jsonObject);
//
//        // 162 为黄浦高血压专病导医机器人
//        // 51 黄浦高血压全科导诊机器人
//        HttpResponse response = HttpRequest.post(qaUrl)
//                .body(req, "application/json")
//                .execute();
//        String body = response.body();
//        log.error("接口请求 req = {}", req);
//        log.error("接口返回 body = {}", body);
//        if (response.getStatus() != 200 || StrUtil.isBlank(body)) {
//            return new R();
//        }
//
//        JSONObject bodyJson = JSON.parseObject(body);
//        String chat = bodyJson.getString("Chat");
//        JSONArray departmentArray = bodyJson.getJSONArray("Department");
//        String medicalType = bodyJson.getString("Medical_type");
//        if(StrUtil.isNotBlank(medicalType) && QA_TYPE.contains(medicalType)){
//            IntelligentAnswerVO answerVO = new IntelligentAnswerVO();
//            answerVO.setAnswer("<a href=\"https://www.sh-sict.com/healthExam\">https://www.sh-sict.com/healthExam</a>");
//            return new R(answerVO);
//        }
//        if (StrUtil.isBlank(chat)) {
//            return new R();
//        }
//
//        chat = chat.replaceAll(QA_RESULT_FLAG, "");
//        if (chat.startsWith("\"") && chat.endsWith("\"")) {
//            chat = chat.substring(1, chat.length() - 1);
//        }
//        currentQa.set(1, chat);
//        current.setQaHistory(JSON.toJSONString(qaHistory));
//        SecurityUtils.updateCurrent(current);
//
//        IntelligentAnswerVO answerVO = new IntelligentAnswerVO();
//        answerVO.setAnswer(chat);
//        if (CollUtil.isNotEmpty(departmentArray)) {
//            answerVO.setDeptName(departmentArray.getString(0));
//        }
//        return new R(answerVO);
//    }

    /**
     * post方式，可以解决特殊符号，过长的文本等问题
     *
     * @return
     */
//    @PostMapping(value = "/completions/stream", produces = MediaType.TEXT_EVENT_STREAM_VALUE)
//    public Flux<String> streamCompletionsPost(@RequestBody Map<String, String> param) {
//        CurrentUser current = SecurityUtils.getCurrentUser();
//        String user = param.get("user");
//        String prompt = param.get("prompt");
//
//        Assert.hasLength(user, "user不能为空");
//        Assert.hasLength(prompt, "prompt不能为空");
//        try {
//            return getChatResponse(param.get("question"), current);
//        } catch (RuntimeException e) {
//            log.warn("e:{}", e.getMessage());
//            return getErrorRes(e.getMessage());
//        } catch (Exception e) {
//            log.error("e:{}", e.getMessage(), e);
//            return getErrorRes(ERROR_MSG);
//        }
//    }
//
//    public Flux<String> getChatResponse(String question, CurrentUser current) {
//        JSONObject params = new JSONObject();
//
//        params.put("username", current.getId());
//        List<String> currentQuestionList = Lists.newArrayListWithCapacity(2);
//        currentQuestionList.add(question);
//        currentQuestionList.add(null);
//        params.put("history", Collections.singletonList(currentQuestionList));
//
//        return webClient.post()
//                .uri("http://localhost:12998/theme-schema/hpStatistics/hos-org-mg-status?endTime=2023-07-23&startTime=2023-07-17")
//                //.header(HttpHeaders.AUTHORIZATION, "Bearer " + "")
//                .syncBody(params.toJSONString())
//                .retrieve()
//                .bodyToFlux(String.class)
//                .onErrorResume(WebClientResponseException.class, ex -> {
//                    HttpStatus status = ex.getStatusCode();
//                    String res = ex.getResponseBodyAsString();
//                    log.error("小欣AI API error: {} {}", status, res);
//                    return Mono.error(new RuntimeException(res));
//                });
//
//    }
//
//    /**
//     * 对sse接口的异常处理
//     * 我这里的建议是不要直接抛出异常中断sse链接，因为这样前端无法获取错误信息，只能获取到链接断开了
//     * 所以建议正常返回数据，把返回的数据中的code设置为非0的值，前端根据code来判断是否是错误信息，参考 @see org.chatgptstream.openai.util.R
//     *
//     * @param msg
//     * @return
//     */
//    private Flux<String> getErrorRes(String msg) {
//        return Flux.create(emitter -> {
//            emitter.next(" ");
//            emitter.next(" ");
//            EXECUTOR.execute(() -> {
//                try {
//                    int time = RANDOM.nextInt(200);
//                    // 请注意！这里加线程池休眠是为了解决一个问题，如果你不需要则删除掉这里线程池就行
//                    // 问题：假如系统使用了nginx负载均衡，然后后端这个接口遇到异常立即断开sse会导致nginx重连，进而重复请求后端
//                    // 所以休眠一下再断开让nginx知道正常连接了，不要重连
//
//                    //不延迟的话nginx会重连sse，导致nginx重复请求后端
//                    Thread.sleep(Math.max(time, 100));
//                } catch (InterruptedException e) {
//                    log.info("e:", e);
//                }
//                emitter.next(JSON.toJSONString(new R<>().error(msg)));
//                emitter.complete();
//            });
//        });
//    }
}
